﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Data.SQLite;
using Brilliant.Data.Common;
using System.IO;
using System.Text.RegularExpressions;

namespace Brilliant.Data.Provider
{
    /// <summary>
    /// SQLite数据访问对象
    /// </summary>
    public class SQLite : DataProviderBase, ISchemaProvider
    {

        public SQLite()
        {
        }

        protected override DbConnection GetConnection()
        {
            return new SQLiteConnection(ConnInfo.ConnectionString);
        }

        protected override DbCommand GetCommand()
        {
            return new SQLiteCommand();
        }

        protected override DbDataAdapter GetDataAdapter()
        {
            return new SQLiteDataAdapter();
        }

        public IList<DboBase> GetDbList()
        {
            string path = ConnInfo.ConnectionString.Substring(0, ConnInfo.ConnectionString.IndexOf(";"));
            path = path.Substring(0, path.LastIndexOf("\\")).Replace("Data Source=", "");
            FileInfo[] files = this.GetFiles(path, "*.db", "*.s*db");
            List<DboBase> list = new List<DboBase>();
            foreach (FileInfo file in files)
            {
                DboBase entity = new DboBase();
                entity.DboId = file.Name;
                entity.DboName = file.Name;
                list.Add(entity);
            }
            return list;
        }

        private FileInfo[] GetFiles(string dirPath, params string[] searchPatterns)
        {
            if (searchPatterns.Length <= 0)
            {
                return null;
            }
            List<FileInfo> list = new List<FileInfo>();
            DirectoryInfo dir = new DirectoryInfo(dirPath);
            foreach (string pattern in searchPatterns)
            {
                list.AddRange(dir.GetFiles(pattern));
            }
            return list.ToArray();
        }

        public IList<DboProc> GetProcList()
        {
            return new List<DboProc>();
        }

        public IList<DboTable> GetTableList()
        {
            SQL sql = new SQL("SELECT name AS DboId,name AS DboName FROM sqlite_master WHERE type='table' ORDER BY name ASC");
            List<DboTable> list = new List<DboTable>();
            using (IDataReader dr = this.ExecDataReader(sql))
            {
                while (dr.Read())
                {
                    DboTable entity = new DboTable();
                    entity.DboId = Convert.ToString(dr["DboId"]);
                    entity.DboName = Convert.ToString(dr["DboName"]);
                    list.Add(entity);
                }
            }
            return list;
        }

        public IList<DboView> GetViewList()
        {
            SQL sql = new SQL("SELECT name AS DboId,name AS DboName FROM sqlite_master WHERE type='view' ORDER BY name ASC");
            List<DboView> list = new List<DboView>();
            using (IDataReader dr = this.ExecDataReader(sql))
            {
                while (dr.Read())
                {
                    DboView entity = new DboView();
                    entity.DboId = Convert.ToString(dr["DboId"]);
                    entity.DboName = Convert.ToString(dr["DboName"]);
                    list.Add(entity);
                }
            }
            return list;
        }

        public SchemaTable GetSchemaInfo(string tableName)
        {
            SchemaTable schemaTable = new SchemaTable();
            schemaTable.TableIndex = 0;
            schemaTable.TableName = TypeMapper.ConvertToUpper(tableName);
            schemaTable.TableNameLower = TypeMapper.ConvertToLower(tableName);
            schemaTable.TableDesc = this.GetTableDesc(tableName);
            schemaTable.TableNameSpace = tableName;
            schemaTable.ColumnList = this.GetColumn(tableName);
            schemaTable.PKList = (from item in schemaTable.ColumnList where item.IsPK select item).ToList();
            schemaTable.FKList = (from item in schemaTable.ColumnList where item.IsFK select item).ToList();
            return schemaTable;
        }

        /// <summary>
        /// 获取表字段列表
        /// </summary>
        public IList<SchemaColumn> GetColumn(string tableName)
        {
            SQL sql = new SQL(GetSchemaQuerySql(tableName));
            IList<SchemaColumn> list = new List<SchemaColumn>();
            using (IDataReader dr = this.ExecDataReader(sql))
            {
                while (dr.Read())
                {
                    SchemaColumn model = new SchemaColumn();
                    string colType = Convert.ToString(dr["Type"]);
                    string colLenght = Regex.Match(colType, "\\(\\d+\\)", RegexOptions.IgnoreCase).Value;
                    model.ColumnIndex = Convert.ToInt32(dr["cid"]);
                    model.ColumnName = TypeMapper.ConvertToUpper(Convert.ToString(dr["name"]));
                    model.ColumnNameLower = TypeMapper.ConvertToLower(model.ColumnName);
                    model.ColumnType = String.IsNullOrEmpty(colLenght) ? colType : colType.Replace(colLenght, "");
                    model.ColumnDefaultValue = Convert.ToString(dr["dflt_value"]);
                    model.ColumnLength = String.IsNullOrEmpty(colLenght) ? 99 : Convert.ToInt32(colLenght.Replace("(", "").Replace(")", ""));
                    model.IsIdentity = false;// Convert.ToString(dr["Identity"]) == "T" ? true : false;
                    model.IsPK = Convert.ToString(dr["pk"]) == "1" ? true : false;
                    model.IsFK = false;//Convert.ToString(dr["pk"]) == "0" ? true : false;
                    model.FkTableName = String.Empty;// Convert.ToString(dr["ForeignKeyTable"]);
                    model.FKTableNameLower = String.Empty;// TypeMap.ConvertToLower(model.FkTableName);
                    model.IsNull = Convert.ToString(dr["notnull"]) == "99" ? true : false;
                    model.ColumnDesc = String.Empty;

                    model.CSharpTypeParser = TypeMapper.MapLanTypeParser(model.ColumnType, LanguageType.CS);
                    model.CSharpType = TypeMapper.MapLanType(model.ColumnType, LanguageType.CS);
                    model.JavaTypeParser = TypeMapper.MapLanTypeParser(model.ColumnType, LanguageType.Java);
                    model.JavaType = TypeMapper.MapLanType(model.ColumnType, LanguageType.Java);
                    model.SqlDbType = TypeMapper.MapSQLType(model.ColumnType, model.ColumnLength);
                    list.Add(model);
                }
            }
            return list;
        }

        /// <summary>
        /// 获取表描述信息
        /// </summary>
        public string GetTableDesc(string tableName)
        {
            return String.Empty;
        }

        /// <summary>
        /// 获取表架构查询语句
        /// </summary>
        /// <param name="tableName">表名称</param>
        /// <returns>表架构查询语句</returns>
        public string GetSchemaQuerySql(string tableName)
        {
            return String.Format("pragma table_info('{0}')", tableName);
        }
    }
}
