﻿using Brilliant.Data.Common;
using Brilliant.Data.Provider;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Brilliant.Data
{
    /// <summary>
    /// 数据库上下文
    /// </summary>
    public class DBContext
    {
        private static readonly Dictionary<string, ConnectionInfo> _pool = new Dictionary<string, ConnectionInfo>();
        private static readonly DBContext _instance = new DBContext();
        private Dictionary<string, ConnectionStringSettings> _settings;
        private ConnectionInfo _currentConnection;
        private string _currentKey;

        /// <summary>
        /// 当前配置文件中所配置连接字符串名称
        /// </summary>
        public static string[] ConnectionStringNames
        {
            get { return _instance._settings.Keys.ToArray(); }
        }

        /// <summary>
        /// 当前连接
        /// </summary>
        public static ConnectionInfo CurrentConnection
        {
            get { return _instance._currentConnection; }
        }

        /// <summary>
        /// 当前连接的键
        /// </summary>
        public static string CurrentKey
        {
            get { return _instance._currentKey; }
        }

        /// <summary>
        /// 数据访问对象实例
        /// </summary>
        public static IDataProvider DataProvider
        {
            get { return CurrentConnection.DataProvider; }
        }

        /// <summary>
        /// 数据表架构访问对象实例
        /// </summary>
        public static ISchemaProvider SchemaProvider
        {
            get { return CurrentConnection.SchemaProvider; }
        }

        /// <summary>
        /// 构造器
        /// </summary>
        private DBContext()
        {
            LoadSettings();
            string defaultSectionName = "default";
            AddConnection(defaultSectionName);
        }

        /// <summary>
        /// 连接到数据源
        /// </summary>
        /// <param name="providerType">数据源类型</param>
        /// <param name="dataSource">服务器地址</param>
        /// <param name="dataBase">默认数据库名称</param>
        /// <param name="uid">用户名</param>
        /// <param name="pwd">密码</param>
        /// <returns>true:连接成功 false:连接失败</returns>
        public static bool Connect(string providerType, string dataSource, string dataBase, string uid, string pwd)
        {
            return _instance.AddConnection(providerType, dataSource, dataBase, uid, pwd);
        }

        /// <summary>
        /// 断开连接
        /// </summary>
        /// <param name="dataSource">数据源</param>
        public static void Disconnect(string dataSource)
        {
            if (_pool.ContainsKey(dataSource))
            {
                _pool.Remove(dataSource);
            }
        }

        /// <summary>
        /// 变更数据源
        /// </summary>
        /// <param name="dataSource">数据源</param>
        public static void ChangeDataSource(string dataSource)
        {
            if (_pool.ContainsKey(dataSource))
            {
                _instance._currentConnection = _pool[dataSource];
                DataProvider.ChangeConnectionString(CurrentConnection.ConnectionString);
            }
        }

        /// <summary>
        /// 变更数据库
        /// </summary>
        /// <param name="dataBase">数据库</param>
        public static void ChangeDataBase(string dataBase)
        {
            if (CurrentConnection != null)
            {
                CurrentConnection.DataBase = dataBase;
                CurrentConnection.ConnectionString = String.Empty;
                DataProvider.ChangeConnectionString(CurrentConnection.ConnectionString);
            }
        }

        /// <summary>
        /// 变更连接字符串节点
        /// </summary>
        /// <param name="sectionName">节点名称</param>
        public static void ChangeSection(string sectionName)
        {
            _instance.AddConnection(sectionName);
            DataProvider.ChangeConnectionString(CurrentConnection.ConnectionString);
        }

        /// <summary>
        /// 加载配置文件
        /// </summary>
        private void LoadSettings()
        {
            _settings = new Dictionary<string, ConnectionStringSettings>();
            foreach (ConnectionStringSettings item in ConfigurationManager.ConnectionStrings)
            {
                string name = item.Name;
                if (name.ToLower().Contains("local"))
                {
                    continue;
                }
                if (!_settings.ContainsKey(item.Name))
                {
                    _settings.Add(item.Name, item);
                }
            }
        }

        /// <summary>
        /// 创建Provider对象
        /// </summary>
        /// <param name="providerName">provider名称</param>
        /// <returns>Provider对象</returns>
        private object CreateProvider(string providerName)
        {
            Type type = null;
            if (!String.IsNullOrEmpty(providerName))
            {
                string namePrefix = "Brilliant.Data.Provider.";
                if (!providerName.Contains(namePrefix))
                {
                    providerName = namePrefix + providerName;
                }
                string assemblyPath = String.Empty;
                if (providerName != typeof(SqlServer).FullName)
                {
                    assemblyPath = String.Format("{0}.dll", providerName);
                    if (!File.Exists(assemblyPath))
                    {
                        assemblyPath = String.Format(@"{0}\bin\{1}.dll", AppDomain.CurrentDomain.BaseDirectory, providerName);
                        if (!File.Exists(assemblyPath))
                        {
                            throw new Exception(String.Format("目标文件\"{0}\"不存在!", assemblyPath));
                        }
                    }
                    Assembly assembly = Assembly.LoadFrom(assemblyPath);
                    type = assembly.GetType(providerName);
                }
                else
                {
                    type = Type.GetType(providerName, true);
                }
            }
            else
            {
                type = typeof(SqlServer);
            }
            return Activator.CreateInstance(type);
        }

        /// <summary>
        /// 添加连接
        /// </summary>
        /// <param name="providerType">数据源类型</param>
        /// <returns>true:连接成功 false:连接失败</returns>
        private bool AddConnection(string providerType)
        {
            if (!_settings.ContainsKey(providerType))
            {
                return false;
            }
            ConnectionInfo connInfo = new ConnectionInfo();
            connInfo.ProviderType = providerType;
            connInfo.ProviderName = ConfigurationManager.ConnectionStrings[providerType].ProviderName;
            connInfo.ConnectionString = ConfigurationManager.ConnectionStrings[providerType].ConnectionString;
            object obj = this.CreateProvider(connInfo.ProviderName);
            connInfo.DataProvider = obj as IDataProvider;
            connInfo.SchemaProvider = obj as ISchemaProvider;
            return this.AddConnection(connInfo);
        }

        /// <summary>
        /// 连接到数据源
        /// </summary>
        /// <param name="providerType">数据源类型</param>
        /// <param name="dataSource">服务器地址</param>
        /// <param name="dataBase">默认数据库名称</param>
        /// <param name="uid">用户名</param>
        /// <param name="pwd">密码</param>
        /// <returns>true:添加成功 false:添加失败</returns>
        private bool AddConnection(string providerType, string dataSource, string dataBase, string uid, string pwd)
        {
            if (!this._settings.ContainsKey(providerType))
            {
                return false;
            }
            ConnectionInfo connInfo = new ConnectionInfo();
            connInfo.ProviderType = providerType;
            connInfo.ProviderName = ConfigurationManager.ConnectionStrings[providerType].ProviderName;
            connInfo.ConnectionStringFormat = ConfigurationManager.ConnectionStrings[providerType].ConnectionString;
            connInfo.DataBase = dataBase;
            connInfo.DataSource = dataSource;
            connInfo.Uid = uid;
            connInfo.Pwd = pwd;
            object obj = this.CreateProvider(connInfo.ProviderName);
            connInfo.DataProvider = obj as IDataProvider;
            connInfo.SchemaProvider = obj as ISchemaProvider;
            return this.AddConnection(connInfo);
        }

        /// <summary>
        /// 添加连接
        /// </summary>
        /// <param name="connInfo">连接字符串</param>
        /// <returns>true:添加成功 false:添加失败</returns>
        private bool AddConnection(ConnectionInfo connInfo)
        {
            string key = String.Format("[{0}]{1}", connInfo.ProviderType, connInfo.DataSource);
            if (connInfo.DataProvider.CheckConnection(connInfo))
            {
                if (_pool.ContainsKey(key))
                {
                    _pool[key] = connInfo;
                }
                else
                {
                    _pool.Add(key, connInfo);
                }
                this._currentConnection = connInfo;
                this._currentKey = key;
                return true;
            }
            return false;
        }
    }
}
